package com.inteligr8.junit4;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.junit.Assert;

/**
 * This class implements JUnit4 assertion methods focused on regular expression matching.
 * 
 * @author brian@inteligr8.com
 */
public class AssertRegex {
	
	/**
	 * This method evaluates the format of the specified value using the
	 * specified regular expression.
	 * 
	 * @param expectedRegex A regular expression
	 * @param actual A text value to evaluate
	 * @see AssertRegex#assertNotMatches(String, String)
	 */
	public static void assertMatches(String expectedRegex, String actual) {
		assertMatches(null, expectedRegex, actual);
	}
	
	/**
	 * This method evaluates the format of the specified value using the
	 * specified regular expression.
	 * 
	 * @param message A message to record on failure
	 * @param expectedRegex A regular expression
	 * @param actual A text value to evaluate
	 * @see AssertRegex#assertNotMatches(String, String, String)
	 */
	public static void assertMatches(String message, String expectedRegex, String actual) {
		Pattern pattern = Pattern.compile(expectedRegex);
		assertMatches(message, pattern, actual);
	}
	
	/**
	 * This method evaluates the format of the specified value using the
	 * specified regular expression.
	 * 
	 * @param expected A regular expression
	 * @param actual A text value to evaluate
	 * @see AssertRegex#assertNotMatches(Pattern, String)
	 */
	public static void assertMatches(Pattern expected, String actual) {
		assertMatches(null, expected, actual);
	}
	
	/**
	 * This method evaluates the format of the specified value using the
	 * specified regular expression.
	 * 
	 * @param message A message to record on failure
	 * @param expected A regular expression
	 * @param actual A text value to evaluate
	 * @see AssertRegex#assertNotMatches(String, Pattern, String)
	 */
	public static void assertMatches(String message, Pattern expected, String actual) {
		assertMatches(message, false, expected, actual);
	}
	
	/**
	 * This method negates the evaluation of the format of the specified value
	 * using the specified regular expression.
	 * 
	 * @param expectedRegex A regular expression
	 * @param actual A text value to evaluate
	 * @see AssertRegex#assertMatches(String, String)
	 */
	public static void assertNotMatches(String expectedRegex, String actual) {
		assertNotMatches(null, expectedRegex, actual);
	}
	
	/**
	 * This method negates the evaluation of the format of the specified value
	 * using the specified regular expression.
	 * 
	 * @param message A message to record on failure
	 * @param expectedRegex A regular expression
	 * @param actual A text value to evaluate
	 * @see AssertRegex#assertMatches(String, String, String)
	 */
	public static void assertNotMatches(String message, String expectedRegex, String actual) {
		Pattern pattern = Pattern.compile(expectedRegex);
		assertNotMatches(message, pattern, actual);
	}
	
	/**
	 * This method negates the evaluation of the format of the specified value
	 * using the specified regular expression.
	 * 
	 * @param expected A regular expression
	 * @param actual A text value to evaluate
	 * @see AssertRegex#assertMatches(Pattern, String)
	 */
	public static void assertNotMatches(Pattern expected, String actual) {
		assertNotMatches(null, expected, actual);
	}
	
	/**
	 * This method negates the evaluation of the format of the specified value
	 * using the specified regular expression.
	 * 
	 * @param message A message to record on failure
	 * @param expected A regular expression
	 * @param actual A text value to evaluate
	 * @see AssertRegex#assertMatches(String, Pattern, String)
	 */
	public static void assertNotMatches(String message, Pattern expected, String actual) {
		assertMatches(message, true, expected, actual);
	}
	
	private static void assertMatches(String message, boolean negate, Pattern expected, String actual) {
		Matcher matcher = expected.matcher(actual);
		if (matcher.matches() == negate) {
			message = message == null ? "" : (message + "; ");
			message += "expression of expected string: <" + expected.toString() + "> but actual string: <" + actual + ">";
			Assert.fail(message);
		}
	}
	
	
	
	/**
	 * This method counts the matches of the specified value against the
	 * specified regular expression.
	 * 
	 * @param expectedRegex A regular expression
	 * @param expectedCount The count of an expected regex match
	 * @param actual A text value to evaluate
	 * @see AssertRegex#assertMatches(String, String)
	 */
	public static void assertFind(String expectedRegex, int expectedCount, String actual) {
		assertFind(null, expectedRegex, expectedCount, actual);
	}
	
	/**
	 * This method counts the matches of the specified value against the
	 * specified regular expression.
	 * 
	 * @param message A message to record on failure
	 * @param expectedRegex A regular expression
	 * @param expectedCount The count of an expected regex match
	 * @param actual A text value to evaluate
	 * @see AssertRegex#assertMatches(String, String, String)
	 */
	public static void assertFind(String message, String expectedRegex, int expectedCount, String actual) {
		Pattern pattern = Pattern.compile(expectedRegex);
		assertFind(message, pattern, expectedCount, actual);
	}
	
	/**
	 * This method counts the matches of the specified value against the
	 * specified regular expression.
	 * 
	 * @param expected A regular expression
	 * @param expectedCount The count of an expected regex match
	 * @param actual A text value to evaluate
	 * @see AssertRegex#assertMatches(Pattern, String)
	 */
	public static void assertFind(Pattern expected, int expectedCount, String actual) {
		assertFind(null, expected, expectedCount, actual);
	}
	
	/**
	 * This method counts the matches of the specified value against the
	 * specified regular expression.
	 * 
	 * @param message A message to record on failure
	 * @param expected A regular expression
	 * @param expectedCount The count of an expected regex match
	 * @param actual A text value to evaluate
	 * @see AssertRegex#assertMatches(Pattern, String)
	 */
	public static void assertFind(String message, Pattern expected, int expectedCount, String actual) {
		Matcher matcher = expected.matcher(actual);
		int count = 0;
		for (count = 0; matcher.find(); count++)
			;
		if (expectedCount < 0) Assert.assertTrue(message, -expectedCount <= count);
		else Assert.assertEquals(message, expectedCount, count);
	}

}
